/*************************************************************************\
* Copyright (c) 1998-2004 The University of Chicago, as Operator of Argonne
* National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
// CChanObj.h : main header file for ChanObj class

#ifndef _INC_CCHANOBJ_H
#define _INC_CCHANOBJ_H

#define CA_PEND_IO_TIME 15.
#define CHAN_OK 1

// Handle exports/imports
#ifdef WIN32
# ifdef CAOBJECT_DLL_SRC
#  define LINK_DLL __declspec(dllexport)
# else
#  define LINK_DLL __declspec(dllimport)
# endif
#else
# define LINK_DLL
#endif

#include <stdarg.h>

// Version
#define CAOBJECT_VERSION        1
#define CAOBJECT_REVISION       0
#define CAOBJECT_MODIFICATION   2
#define CAOBJECT_PATCH_LEVEL   0
#define CAOBJECT_VERSION_STRING "CaObject Version 1.0.2.0"

class LINK_DLL CChanObj;

typedef enum {monitor_off,monitor_req,monitor_on} ChanMonitorState;

typedef int (*ChanErrorHandler)(const char *fmt, ...);
typedef void (*ChanValueChangedCB)(CChanObj *chan);
typedef void (*ChanConnectionChangedCB)(CChanObj *chan);

#include <cadef.h>
#include <epicsVersion.h>

class LINK_DLL CChanObj {
  public:
    CChanObj();
    ~CChanObj();
    int connect(const char *name);
    int connectCallback(const char *name);
    void disconnect(void);
    int startMonitor(void);
    int stopMonitor(void);
    int get(void);
    int put(const char *newval);
    int getCallback(void);
    int putCallback(const char *newval);

    static void addFDRegistration(CAFDHANDLER *handler, void *pUser=NULL);
    static void poll(void) { ca_poll(); }
    static const char *getVersion(void) { return CAOBJECT_VERSION_STRING; }
    static const char *getEpicsVersion(void) { return EPICS_VERSION_STRING; }

    const char *getName(void) const { return ca_name(Chid); }
    const char *getValue(void) const { return value; }
    int getMonitored(void) const { return monitored; }
    int getValueChanged(void) const { return valueChanged; }
    void clearValueChanged(void) { valueChanged=0; }
    void setValueChanged(int vc) { valueChanged=vc; }
    void *getUserData(void) const { return userData; }
    void setUserData(void *userDataIn) { userData=userDataIn; }
    short getState(void) { return ca_state(Chid); }
    double getPendIOTime(void) const { return ca_pend_io_time; }
    void setPendIOTime(double sec) { ca_pend_io_time=sec; }

    int getConnected(void) const { return connected; }
    chid getChid(void) const { return Chid; }
    unsigned long getRefCount(void) const { return refCount; }

    void setErrorHandler(ChanErrorHandler handler)
      {	errHandler=handler;}
    ChanErrorHandler getErrorHandler(void) const
      {	return errHandler; }
    ChanErrorHandler restoreErrorHandler(void)
      {	return (errHandler=errMsg); }

    void setUserValueChangedCB(ChanValueChangedCB cb)
      {	userValueChangedCB=cb; }
    ChanValueChangedCB getUserValueChangedCB(void) const
      {	return userValueChangedCB; }

    void setUserConnectionChangedCB(ChanConnectionChangedCB cb)
      {	userConnectionChangedCB=cb; }
    ChanConnectionChangedCB getUserConnectionChangedCB(void) const
      {	return userConnectionChangedCB; }

  private:
    int valueChanged;
    char value[MAX_STRING_SIZE+1];
    void *userData;
    chid Chid;
    evid valueChangedEvid;
    double ca_pend_io_time;
    int connected;
    ChanMonitorState monitored;

    static int caStarted;
    static unsigned long refCount;

    ChanErrorHandler errHandler;
    ChanValueChangedCB userValueChangedCB;
    ChanConnectionChangedCB userConnectionChangedCB;

    int taskInit(void);
    int taskExit(void);

  public:
    static int errMsg(const char *fmt, ...);
    static void exceptionCB(struct exception_handler_args arg);
    static void valueChangedCB(struct event_handler_args args);
    static void connectionChangedCB(
      struct connection_handler_args args);
};

#endif // _INC_CCHANOBJ_H
